@@ -61,6 +61,14 @@ eng_python <- function(options) {
     paste(snippet[nzchar(snippet)], collapse = "\n")
   }
   
+  # helper function for running a snippet of code and capturing output
+  run <- function(snippet) {
+    output <- py_capture_output(py_run_string(snippet, convert = FALSE))
+    if (nzchar(output))
+      output <- sub("\n$", "", output)
+    output
+  }
+  
   # extract the code to be run -- we'll attempt to run the code line by line
   # and detect changes so that we can interleave code and output (similar to
   # what one sees when executing an R chunk in knitr). to wit, we'll do our
@@ -96,28 +104,51 @@ eng_python <- function(options) {
   
   for (range in ranges) {
     
-    # evaluate current chunk of code
+    # extract code to be run
     snippet <- extract(code, range)
-    captured <- py_capture_output(
-      py_run_string(snippet, convert = FALSE)
-    )
+    
+    # run code and capture output (leave output
+    # empty for 'eval = FALSE' case
+    captured <- ""
+    if (!identical(options$eval, FALSE)) {
+      if (is.numeric(options$eval))
+        warning("numeric 'eval' chunk option not supported by reticulate engine")
+      
+      # error=TRUE implies that errors should be captured and converted
+      # into output messages
+      if (identical(options$error, TRUE)) {
+        tryCatch(
+          captured <- run(snippet),
+          error = function(e) {
+            captured <<- conditionMessage(e)
+          }
+        )
+      } else {
+        captured <- run(snippet)
+      }
+    }
     
     if (nzchar(captured) || length(context$pending_plots)) {
       
-      # append pending source to outputs
-      outputs[[length(outputs) + 1]] <- structure(
-        list(src = extract(code, c(pending_source_index, range[2]))),
-        class = "source"
-      )
+      # append pending source to outputs (respecting 'echo' option)
+      if (!identical(options$echo, FALSE)) {
+        if (is.numeric(options$echo))
+          warning("numeric 'echo' chunk option not supported by reticulate engine")
+        extracted <- extract(code, c(pending_source_index, range[2]))
+        output <- structure(list(src = extracted), class = "source")
+        outputs[[length(outputs) + 1]] <- output
+      }
       
       # append captured outputs
-      if (nzchar(captured))
+      if (nzchar(captured) && isTRUE(options$include))
         outputs[[length(outputs) + 1]] <- captured
       
       # append captured images / figures
       if (length(context$pending_plots)) {
-        for (plot in context$pending_plots)
-          outputs[[length(outputs) + 1]] <- plot
+        if (isTRUE(options$include)) {
+          for (plot in context$pending_plots)
+            outputs[[length(outputs) + 1]] <- plot
+        }
         context$pending_plots <- list()
       }
       
@@ -127,7 +158,9 @@ eng_python <- function(options) {
   }
   
   # if we have leftover input, add that now
-  if (pending_source_index <= n) {
+  if (!identical(options$echo, FALSE) && pending_source_index <= n) {
+    if (is.numeric(options$echo))
+      warning("numeric 'echo' chunk option not supported by reticulate engine")
     leftover <- extract(code, c(pending_source_index, n))
     outputs[[length(outputs) + 1]] <- structure(
       list(src = leftover),
@@ -137,10 +170,7 @@ eng_python <- function(options) {
   
   eng_python_synchronize_after()
   
-  # TODO: development version of knitr supplies new 'engine_output()'
-  # interface -- use that when it's on CRAN
-  # https://github.com/yihui/knitr/commit/71bfd8796d485ed7bb9db0920acdf02464b3df9a
-  wrap <- yoink("knitr", "wrap")
+  wrap <- getOption("reticulate.engine.wrap", eng_python_wrap)
   wrap(outputs, options)
   
 }
@@ -153,6 +183,14 @@ eng_python_initialize <- function(options, context, envir) {
   eng_python_initialize_matplotlib(options, context, envir)
 }
 
+eng_python_matplotlib_show <- function(plt, options) {
+  plot_counter <- yoink("knitr", "plot_counter")
+  path <- knitr::fig_path(options$dev, number = plot_counter())
+  dir.create(dirname(path), recursive = TRUE, showWarnings = FALSE)
+  plt$savefig(path, dpi = options$dpi)
+  knitr::include_graphics(path)
+}
+
 eng_python_initialize_matplotlib <- function(options,
                                              context,
                                              envir)
@@ -166,8 +204,8 @@ eng_python_initialize_matplotlib <- function(options,
   matplotlib <- import("matplotlib", convert = FALSE)
   plt <- matplotlib$pyplot
   
-  # rudely steal 'plot_counter' (used below), and reset
-  # it when we're done
+  # rudely steal 'plot_counter' (used by default 'show()' implementation below)
+  # and then reset the counter when we're done
   plot_counter <- yoink("knitr", "plot_counter")
   defer(plot_counter(reset = TRUE), envir = envir)
   
@@ -175,15 +213,9 @@ eng_python_initialize_matplotlib <- function(options,
   show <- plt$show
   defer(plt$show <- show, envir = envir)
   plt$show <- function(...) {
-    
-    # write plot to file
-    path <- knitr::fig_path(options$dev, number = plot_counter())
-    dir.create(dirname(path), recursive = TRUE, showWarnings = FALSE)
-    plt$savefig(path, dpi = options$dpi)
-    
-    # return as a knitr image path
-    context$pending_plots[[length(context$pending_plots) + 1]] <<-
-      knitr::include_graphics(path)
+    hook <- getOption("reticulate.engine.matplotlib.show", eng_python_matplotlib_show)
+    graphic <- hook(plt, options)
+    context$pending_plots[[length(context$pending_plots) + 1]] <<- graphic
   }
   
   # set up figure dimensions
@@ -202,8 +234,11 @@ eng_python_synchronize_before <- function() {
   R <- main$R
   
   # extract active knit environment
-  .knitEnv <- yoink("knitr", ".knitEnv")
-  envir <- .knitEnv$knit_global
+  envir <- getOption("reticulate.engine.environment")
+  if (is.null(envir)) {
+    .knitEnv <- yoink("knitr", ".knitEnv")
+    envir <- .knitEnv$knit_global
+  }
   
   # define the getters, setters we'll attach to the Python class
   getter <- function(self, code) {
@@ -224,5 +259,12 @@ eng_python_synchronize_before <- function() {
 }
 
 # synchronize objects Python -> R
-eng_python_synchronize_after <- function() {
-}
\ No newline at end of file
+eng_python_synchronize_after <- function() {}
+
+eng_python_wrap <- function(outputs, options) {
+  # TODO: development version of knitr supplies new 'engine_output()'
+  # interface -- use that when it's on CRAN
+  # https://github.com/yihui/knitr/commit/71bfd8796d485ed7bb9db0920acdf02464b3df9a
+  wrap <- yoink("knitr", "wrap")
+  wrap(outputs, options)
+}
