@@ -10,6 +10,7 @@ using namespace Rcpp;
 #include "tinythread.h"
 
 #include <fstream>
+#include <time.h>
 
 using namespace libpython;
 
@@ -1397,6 +1398,46 @@ void py_activate_virtualenv(const std::string& script)
     stop(py_fetch_error());
 }
 
+void trace_print(int threadId, PyFrameObject *frame) {
+  std::string tracemsg = "";
+  while (NULL != frame) {
+    std::string filename = as_std_string(frame->f_code->co_filename);
+    std::string funcname = as_std_string(frame->f_code->co_name);
+    tracemsg = funcname + " " + tracemsg;
+    
+    frame = frame->f_back;
+  }
+  
+  tracemsg = "THREAD: [" + tracemsg + "]\n";
+  PySys_WriteStderr(tracemsg.c_str());
+}
+
+void trace_thread_main(void* aArg) {
+  using namespace tthread;
+  
+  int* tracems = (int*)aArg;
+  
+  while (true) {
+    PyGILState_STATE gstate;
+    gstate = PyGILState_Ensure();
+    
+    PyThreadState* pState = PyGILState_GetThisThreadState();
+    
+    while (pState != NULL) {
+      trace_print(pState->thread_id, pState->frame);
+      pState = PyThreadState_Next(pState);
+    }
+    
+    PyGILState_Release(gstate);
+    
+    this_thread::sleep_for(chrono::milliseconds(*tracems));
+  }
+}
+
+tthread::thread* ptrace_thread;
+void trace_thread_init(int tracems) {
+  ptrace_thread = new tthread::thread(trace_thread_main, &tracems);
+}
 
 // [[Rcpp::export]]
 void py_initialize(const std::string& python,
@@ -1479,6 +1520,13 @@ void py_initialize(const std::string& python,
   else
     s_numpy_load_error = numpy_load_error;
   
+  // initialize trace
+  Function sysGetEnv("Sys.getenv");
+  std::string tracems_env = as<std::string>(sysGetEnv("RETICULATE_DUMP_STACK_TRACE", 0));
+  int tracems = ::atoi(tracems_env.c_str());
+  if (tracems > 0) 
+    trace_thread_init(tracems);
+  
   // poll for events while executing python code
   event_loop::initialize();
 }
